# new, revised MakePolys

import arcpy, sys, os.path

versionString = 'NCGMP09v1.1_MakePolys2_Arc10.1.py, version of 11 September 2013'
editPrefixes = ('xxx','edit_','errors_')
debug1 = False
debug2 = False
debug3 = True
default = '#'

# utility functions

def addMsgAndPrint(msg, severity=0): 
    # prints msg to screen and adds msg to the geoprocessor (in case this is run as a tool) 
    try: 
      for string in msg.split('\n'): 
        # Add appropriate geoprocessing message 
        if severity == 0: 
            arcpy.AddMessage(string) 
        elif severity == 1: 
            arcpy.AddWarning(string) 
        elif severity == 2: 
            arcpy.AddError(string) 
    except: 
        pass
	
def testAndDelete(fc):
    if arcpy.Exists(fc):
        if debug1: addMsgAndPrint('    deleting '+fc)
        arcpy.Delete_management(fc)
    return

def quitInError():
    addMsgAndPrint('Forcing exit with error\n')
    raise arcpy.ExecuteError

def nRows(aTable):
    return int(arcpy.GetCount_management(aTable).getOutput(0))

def getCaf(fds):
    if os.path.basename(fds) == 'CorrelationOfMapUnits':
        return 'CMULines'
    else:
        oldWS = arcpy.env.workspace
        arcpy.env.workspace = fds
        fcs = arcpy.ListFeatureClasses()
        arcpy.env.workspace = oldWS
        cafs = []
        for fc in fcs:
            if fc.find('ContactsAndFaults') > -1:
                cafs.append(fc)
        for fc in cafs:
            for pfx in editPrefixes:
                if fc.find(pfx) > -1:  # no prefix
                    cafs.remove(fc)
        cafs2 = []
        for fc in cafs:
            if fc[-17:] == 'ContactsAndFaults':
                cafs2.append(fc)
        #addMsgAndPrint(str(cafs))
        if len(cafs2) <> 1:
            addMsgAndPrint('  Cannot resolve ContactsAndFaults feature class in feature dataset')
            addMsgAndPrint('    '+fds)
            addMsgAndPrint('    '+str(cafs2))
            quitInError()
        return cafs2[0]

def getNameToken(fds):
    if os.path.basename(fds) == 'CorrelationOfMapUnits':
        return 'CMU'
    else:
        caf = getCaf(fds)
        return caf.replace('ContactsAndFaults','')
    
def getMup(fds):
    if os.path.basename(fds) == 'CorrelationOfMapUnits':
        return 'CMUPolys'
    else:
        caf = getCaf(fds)
        return caf.replace('ContactsAndFaults','MapUnitPolys')

def writeLayerNames(lyr):
    if lyr.supports('datasetName'): addMsgAndPrint('datasetName: '+lyr.datasetName)
    if lyr.supports('dataSource'):  addMsgAndPrint(' dataSource: '+lyr.dataSource)
    if lyr.supports('longName'):    addMsgAndPrint('   longName: '+lyr.longName)
    if lyr.supports('name'):        addMsgAndPrint('       name: '+lyr.name)
    
def getSaveName(fc):
    # fc is entire pathname
    # builds new, unused name in form oldNameNNN
    oldWS = arcpy.env.workspace
    arcpy.env.workspace = os.path.dirname(fc)
    shortFc = os.path.basename(fc)
    pfcs = arcpy.ListFeatureClasses(shortFc+'*')
    if debug1: addMsgAndPrint(str(pfcs))
    maxN = 0
    for pfc in pfcs:
        try:
            n = int(pfc.replace(shortFc,''))
            if n > maxN:
                maxN = n
        except:
            pass
    saveName = fc+str(maxN+1).zfill(3)
    arcpy.env.workspace = oldWS
    if debug1:
        addMsgAndPrint('fc = '+fc)
        addMsgAndPrint('saveName = '+saveName)
    return saveName

def findLyr(lname):
    if debug1: addMsgAndPrint('lname = '+lname)
    mxd = arcpy.mapping.MapDocument('CURRENT')
    for df in arcpy.mapping.ListDataFrames(mxd):
        lList = arcpy.mapping.ListLayers(mxd, '*', df)
        if debug1:
            for lyr in lList:
                writeLayerNames(lyr)
        for lyr in lList:
            # either layer is a group, datasetName is not supported, and we match lyr.name
            # or (and) we match datasetName, which cannot be aliased as lyr.name may be
            if (lyr.supports('dataSource') and lyr.dataSource == lname) \
                    or (not lyr.supports('dataSource') and lyr.name == lname):
                pos = lList.index(lyr)
                if pos == 0:
                    refLyr = lList[pos + 1]
                    insertPos = "BEFORE"
                else:
                    refLyr = lList[pos - 1]
                    insertPos = "AFTER"
                return [lyr, df, refLyr, insertPos]

def analyzeRowSet(rowSet,badPolyList,badLabelList):
    #fields = ['FID_MapUnitPolys','MapUnit','IdentityConfidence','DataSourceID','OBJECTID','OldPoly']
    # if len(rowSet) == 2, MapUnit, IdCon, and DASID are the same, and row[0].OldPoly + row[1] == 1, all is OK
    if len(rowSet) == 2 and rowSet[0][1] == rowSet[1][1] and rowSet[0][2] == rowSet[1][2] and rowSet[0][3] == rowSet[1][3] and rowSet[0][4] <> rowSet[1][4]:
        #addMsgAndPrint('    all is well')
        pass
    # else add FID_MapUnitPolys to badPolyList. For rows with OldPoly = 1, add OBJECTID to badLabelList
    else:
        badPolyList.append(rowSet[0][0])
        for row in rowSet:
            badLabelList.append(row[4])
    return

################################################

addMsgAndPrint('  '+versionString)
# get inFds, saveMUP
fds = sys.argv[1]
saveMUP = False
if sys.argv[2] == 'true':
    saveMUP = True
if sys.argv[3] == '#':
    layerRepository = os.path.dirname(os.path.dirname(fds))
else:
    layerRepository = sys.argv[3]
  
# check for existence of fds
if not arcpy.Exists(fds):
    addMsgAndPrint('Feature dataset '+fds+ 'does not seem to exist.')
    quitInError()
# check for schema lock
if not arcpy.TestSchemaLock(fds):
    addMsgAndPrint('Feature dataset '+fds+' is locked!')
    quitInError()

# get caf, mup, nameToken
shortCaf = getCaf(fds)
caf = os.path.join(fds,shortCaf)
shortMup = getMup(fds)
mup = os.path.join(fds,shortMup)
nameToken = getNameToken(fds)

# check that either MapUnitPolys or MapUnitPoints exists.
#  one or other is needed to define attributes for new MapUnitPolys fc

if not ( arcpy.Exists(mup) or arcpy.Exists(os.path.join(fds,nameToken+'MapUnitPoints')) ):
    addMsgAndPrint(' \nNo MapUnitPolys or MapUnitPoints feature class present.')
    addMsgAndPrint('  One is needed as template for new MapUnitPolys feature class.\n ')
    quitInError()                                       

# check for topology class that involves polys
arcpy.env.workspace = fds
topologies = arcpy.ListDatasets('*','Topology')
for topol in topologies:
    if shortMup in arcpy.Describe(topol).featureClassNames:
        addMsgAndPrint('Cannot delete '+shortMup)
        addMsgAndPrint('  because it is part of topology class '+topol+'.')
        addMsgAndPrint('  Delete topology (or remove rule) before running this script.')
        quitInError()

badLabels = os.path.join(fds,'errors_'+nameToken+'multilabels')
badPolys = os.path.join(fds,'errors_'+nameToken+'multilabelPolys')
blankPolys = os.path.join(fds,'errors_'+nameToken+'unlabeledPolys')
excessContacts = os.path.join(fds,'errors_'+nameToken+'excessContacts')
idCAF = os.path.join(fds,'edit_'+nameToken+'CAFwithPolys')

# find and remove layers in active window that involve editLayers
addMsgAndPrint('  Saving selected layers and removing them from current data frame')
savedLayers = []
layerN = 1
for aLyr in mup,badLabels,badPolys,blankPolys,excessContacts,idCAF:
    layerFound = True
    while layerFound:  # repeat to look for multiple layers with fc of interest
        try:
            lyr,df,refLyr,insertPos = findLyr(aLyr)
            # if layer is part of a layer group, get layer group instead
            while lyr.longName.find('\\') > 0:
                groupLyrName = lyr.longName[:lyr.longName.find('\\')]
                lyr,df,refLyr,insertPos = findLyr(groupLyrName)
            # WHY OH WHY do we let Windoze programmers build our tools?
            lyrName = lyr.name.replace('\\','_')
            lyrPath = os.path.join(layerRepository, lyrName + str(layerN) + '.lyr')
            #save to a layer file on disk so that customizations can be retrieved layer
            if arcpy.Exists(lyrPath):
                os.remove(lyrPath)
            try:
                addMsgAndPrint('    saving layer '+str(lyr))
                arcpy.SaveToLayerFile_management(lyr, lyrPath, "RELATIVE")
            except:
                addMsgAndPrint('    failed to save lyr '+str(lyr)+ ' to path '+lyrPath)
                raise arcpy.ExecuteError
            #and now remove the layer
            arcpy.mapping.RemoveLayer(df, lyr)
            savedLayers.append([lyrPath,df,refLyr,insertPos,lyr])
            addMsgAndPrint('     layer '+lyrName+' saved and removed from data frame')
            layerN = layerN+1
        except:  # crashes because there is no matching layer
            layerFound = False
    
if debug1: addMsgAndPrint('savedLayers ='+str(savedLayers))

xxPolys = 'xxxpolys'
xxLabels = 'xxxlabels'
tempLabels = 'xxxtlabels'
tempLabels2 = 'xxxtlabels2'
cafLayer = 'cafLayer'

arcpy.env.workspace = fds
# delete any existing output and scratchlayers
for fc in tempLabels,tempLabels2,cafLayer,xxLabels,xxPolys,badPolys,badLabels,blankPolys,idCAF,excessContacts:
    testAndDelete(fc)
# if mup exists, FeaturesToPoint() to make temporary labels
if arcpy.Exists(mup):
    addMsgAndPrint('  Making temporary labels from existing mapunit polygons')
    arcpy.FeatureToPoint_management(mup,tempLabels,'INSIDE')
    # select MapUnit <> '' to layer
    sqlQuery = arcpy.AddFieldDelimiters(fds,'MapUnit') +  " <> ''"
    arcpy.MakeFeatureLayer_management(tempLabels,tempLabels2,sqlQuery)
    # flag polyPoints as MUP-derived
    arcpy.AddField_management (tempLabels2, 'OldPoly', 'SHORT')
    arcpy.CalculateField_management (tempLabels2, 'OldPoly', 1)
    # if label points exist, append to polyPoints
    if arcpy.Exists(nameToken+'MapUnitPoints'):
        addMsgAndPrint('  Appending MapUnitPoints to temporary labels')
        arcpy.Append_management('MapUnitPoints',tempLabels2,'NO_TEST')
    # if saveMUP, copy mup to mupNNN
    if saveMUP:
        saveName = getSaveName(mup)
        addMsgAndPrint('  Saving '+shortMup+' to '+os.path.basename(saveName))
        try:
            arcpy.Copy_management(mup,saveName)
        except:
            addMsgAndPrint(" arcpy.Copy_management(mup,saveName) failed. Maybe you need to close ArcMap?")
            raise arcpy.ExecuteError

# make layer view of inCaf without concealed lines
sqlQuery = arcpy.AddFieldDelimiters(fds,'IsConcealed') + " NOT IN ('Y','y')"
arcpy.MakeFeatureLayer_management(caf,cafLayer,sqlQuery)

# feature to polys, with polyPoints as labels
addMsgAndPrint('  Making new MapUnitPolys')
testAndDelete(mup)
if arcpy.Exists(tempLabels2) and nRows(tempLabels2) > 0:
    arcpy.FeatureToPolygon_management('cafLayer',mup,'','ATTRIBUTES',tempLabels2)
else:
    arcpy.FeatureToPolygon_management('cafLayer',mup)

# identity polyPoints with polys
addMsgAndPrint('  Intersecting (IDENTITY) points and MapUnitPolys')
arcpy.Identity_analysis(tempLabels2, mup, xxLabels)

# sort polyPoints and find errors
badPolyList = []
badLabelList = []
oldFID = -9
rowSet = []
fields = ['FID_MapUnitPolys','MapUnit','IdentityConfidence','DataSourceID','OBJECTID','OldPoly']
addMsgAndPrint('  Iterating through all labels ')
for row in sorted(arcpy.da.SearchCursor(xxLabels, fields)):
    if row[0] == oldFID:
        # add row to rowSet
        rowSet.append(row)
    else:
        #pass rowSet to analysis function
        if len(rowSet) > 1:
            analyzeRowSet(rowSet,badPolyList,badLabelList)
        #start new rowSet
        rowSet = [row]
        oldFID = row[0]
if len(rowSet) > 1: analyzeRowSet(rowSet,badPolyList,badLabelList)   
addMsgAndPrint('    '+str(len(badPolyList))+' multilabel polys and '+str(len(badLabelList))+ ' bad labels\n  ')

# make badLabels
addMsgAndPrint('  Getting bad labels')
arcpy.AddField_management(xxLabels,'MultipleLabel','TEXT',default,default,5)
fields = ['OBJECTID','MultipleLabel']
with arcpy.da.UpdateCursor(xxLabels, fields) as cursor:
    for row in cursor:
        if row[0] in badLabelList:
            row[1] = 'YES'
            cursor.updateRow(row)
query = arcpy.AddFieldDelimiters(fds,'MultipleLabel') + " = 'YES'"
arcpy.Select_analysis(xxLabels,badLabels,query)
addMsgAndPrint('    '+str(nRows(badLabels))+' rows in '+os.path.basename(badLabels))

# make badPolys
addMsgAndPrint('  Getting multi-label polys')
arcpy.Copy_management(mup, os.path.join(fds,xxPolys))
arcpy.AddField_management(xxPolys,'MultipleLabels','TEXT',default,default,5)
OidName = arcpy.Describe(xxPolys).OIDFieldName
fields = [OidName,'MultipleLabels']
with arcpy.da.UpdateCursor(xxPolys, fields) as cursor:
    for row in cursor:
        if row[0] in badPolyList:
            row[1] = 'YES'
            cursor.updateRow(row)
query = arcpy.AddFieldDelimiters(fds,'MultipleLabels') + " = 'YES'"
arcpy.Select_analysis(xxPolys,badPolys,query)
addMsgAndPrint('    '+str(nRows(badPolys))+' rows in '+os.path.basename(badPolys))

# make blankPolys
addMsgAndPrint('  Getting unlabeled polys')
query = arcpy.AddFieldDelimiters(fds,'MapUnit') + " = ''"
arcpy.Select_analysis (mup, blankPolys, query)
addMsgAndPrint('    '+str(nRows(blankPolys))+' rows in '+os.path.basename(blankPolys))

# make idCAF
addMsgAndPrint('  Getting excess contacts')
arcpy.Identity_analysis(caf,mup,idCAF,'ALL','','KEEP_RELATIONSHIPS')
# select those arcs that are contacts, are not concealed, and have same map unit in both sides
query = arcpy.AddFieldDelimiters(fds,'Type') + " LIKE 'contact' and " + arcpy.AddFieldDelimiters(fds,'IsConcealed')
query = query + " = 'N' and "+arcpy.AddFieldDelimiters(fds,'Left_MapUnit')+" = "+arcpy.AddFieldDelimiters(fds,'Right_MapUnit')
arcpy.Select_analysis(idCAF,excessContacts,query)
addMsgAndPrint('    '+str(nRows(excessContacts))+' rows in '+os.path.basename(excessContacts))
 
# restore saved layers
addMsgAndPrint('  Restoring saved layers to current data frame')
savedLayers.reverse()
for savedLayer in savedLayers:
    lyrPath,dataFrame,refLyr,insertPos,lyr = savedLayer
    addMsgAndPrint('    layer '+lyr.name)
    if debug2: addMsgAndPrint('      '+str(lyrPath)+' '+str(dataFrame)+' '+str(refLyr)+' '+str(insertPos))
    addLyr = arcpy.mapping.Layer(lyrPath)
    arcpy.mapping.AddLayer(dataFrame, addLyr)
    # if refLyr is part of a layer group, substiture layer group 
    refLyrName = refLyr.longName
    if debug2: addMsgAndPrint(refLyrName)
    while refLyrName.find('\\') > 0:
            groupLyrName = refLyrName[:refLyrName.find('\\')]
            if debug2: addMsgAndPrint(groupLyrName)
            refLyr = findLyr(groupLyrName)[0]
            refLyrName = refLyr.longName
    try:
        addMsgAndPrint('    '+str(addLyr))
        arcpy.mapping.InsertLayer(dataFrame, refLyr, addLyr, insertPos)
        arcpy.Delete_management(lyrPath)
    except:
        addMsgAndPrint('    failed to insert '+str(addLyr)+' '+insertPos+' refLyr '+str(refLyr))
        if debug3:
            mxd = arcpy.mapping.MapDocument('CURRENT')
            for df in arcpy.mapping.ListDataFrames(mxd):
                lList = arcpy.mapping.ListLayers(mxd, '*', df)
                addMsgAndPrint(' refLyr = '+str(refLyr))
                for lyr in lList:
                    addMsgAndPrint('      '+str(lyr)+', matches refLyr = '+str(lyr == refLyr))
                    if str(lyr) == str(refLyr):
                        writeLayerNames(lyr)
                        writeLayerNames(refLyr)
                        writeLayerNames(addLyr)            

# delete scratchlayers
for fc in xxLabels,xxPolys,tempLabels,tempLabels2,cafLayer:
    testAndDelete(fc)

addMsgAndPrint('  \nCompleted successfully\n ')
quitInError()

